# coding: UTF-8
from __future__ import unicode_literals
from pypugly.text import dedent, indent, safe_split, match_any, format_it


def test_dedent0():
    string = dedent('oneline')
    assert string == 'oneline'


def test_dedent1():
    string = dedent(
        """
        oneline
        """
    )
    assert string == 'oneline'


def test_dedent2():
    string = dedent(
        """
        oneline
        twoline
        """
    )
    assert string == 'oneline\ntwoline'


def test_dedent3():
    string = dedent(
        """
        oneline
            tabbed
        """
    )
    assert string == 'oneline\n    tabbed'


def test_dedent4():
    string = dedent(
        """
        oneline
            tabbed
        detabbed
        """
    )
    assert string == 'oneline\n    tabbed\ndetabbed'


def test_dedent5():
    string = dedent(
        """
        oneline
        """,
        ignore_first_linebreak=False
    )
    assert string == '\noneline'


def test_dedent6():
    string = dedent(
        """
        oneline
        """,
        ignore_last_linebreak=False
    )
    assert string == 'oneline\n'


def test_dedent7():
    """
    Test a string that has an 'empty line' with 4 spaces above indent level
    """
    # Using a trick to avoid auto-format to remove the empty spaces.
    string = dedent(
        """
        line
        %s
        other_line
        """ % '    '
    )
    assert string == 'line\n    \nother_line'


def test_dedent8():
    """
    Test not the first line in the right indent.
    """
    string = dedent(
        """
            alpha
          bravo
        charlie
        """
    )
    assert string == '    alpha\n  bravo\ncharlie'


def test_dedent9():
    """
    Test behavior when using \t at the start of a string. .. seealso:: BEN-21 @ JIRA
    """
    string = dedent(
        """
            alpha
        \tbravo
        """
    )
    assert string == '    alpha\n\tbravo'


def test_dedent10():
    """
    Checking how dedent handles empty lines at the end of string without parameters.
    """
    string = dedent(
        """
        alpha
        """
    )
    assert string == 'alpha'
    string = dedent(
        """
        alpha

        """
    )
    assert string == 'alpha\n'
    string = dedent(
        """
        alpha


        """
    )
    assert string == 'alpha\n\n'


def test_dedent11():
    """
    Check calling dedent more than once.
    """
    string = dedent(
        """
        alpha
        bravo
        charlie
        """
    )
    assert string == 'alpha\nbravo\ncharlie'
    string = dedent(string)
    assert string == 'alpha\nbravo\ncharlie'
    string = dedent(string)
    assert string == 'alpha\nbravo\ncharlie'


def test_indent():
    assert indent('alpha') == '    alpha'

    assert indent('alpha', indent=2) == '        alpha'
    assert indent('alpha', indentation='...') == '...alpha'

    # If the original text ended with '\n' the resulting text must also end with '\n'
    assert indent('alpha\n') == '    alpha\n'

    # If the original text ended with '\n\n' the resulting text must also end with '\n\n'
    # Empty lines are not indented.
    assert indent('alpha\n\n') == '    alpha\n\n'

    # Empty lines are not indented nor cleared.
    assert indent('alpha\n  \ncharlie') == '    alpha\n  \n    charlie'

    # Empty lines are not indented nor cleared.
    assert indent(['alpha', 'bravo']) == '    alpha\n    bravo\n'

    # Multi-line test.
    assert indent('alpha\nbravo\ncharlie') == '    alpha\n    bravo\n    charlie'


def test_safe_split():
    assert safe_split('alpha', ' ') == ['alpha']
    assert safe_split('alpha bravo', ' ') == ['alpha', 'bravo']
    assert safe_split('alpha bravo charlie', ' ') == ['alpha', 'bravo', 'charlie']

    assert safe_split('alpha', ' ', 1) == ['alpha', '']
    assert safe_split('alpha bravo', ' ', 1) == ['alpha', 'bravo']
    assert safe_split('alpha bravo charlie', ' ', 1) == ['alpha', 'bravo charlie']

    assert safe_split('alpha', ' ', 1, default=9) == ['alpha', 9]
    assert safe_split('alpha bravo', ' ', 1, default=9) == ['alpha', 'bravo']
    assert safe_split('alpha bravo charlie', ' ', 1, default=9) == ['alpha', 'bravo charlie']

    assert safe_split('alpha', ' ', 2) == ['alpha', '', '']
    assert safe_split('alpha bravo', ' ', 2) == ['alpha', 'bravo', '']
    assert safe_split('alpha bravo charlie', ' ', 2) == ['alpha', 'bravo', 'charlie']

    assert safe_split('alpha:bravo:charlie', ':', 1) == ['alpha', 'bravo:charlie']
    assert safe_split('alpha:bravo:charlie', ':', 1, reversed=True) == ['alpha:bravo', 'charlie']

    assert safe_split('alpha', ':', 1, []) == ['alpha', []]
    assert safe_split('alpha', ':', 1, [], reversed=True) == [[], 'alpha']


def test_format_id():
    item1 = 'a'
    item2 = 'b'
    my_list = [item1, item2]

    assert format_it(my_list) == "['a', 'b']"


def test_match_any():
    assert match_any('alpha', ['alpha', 'bravo']) == True
    assert match_any('bravo', ['alpha', 'bravo']) == True
    assert match_any('charlie', ['alpha', 'bravo']) == False
    assert match_any('one:alpha', ['one:.*', ]) == True
    assert match_any('two:alpha', ['one:.*', ]) == False



from __future__ import unicode_literals, print_function
from pypeg2 import *

number = re.compile(r"\d+")
symbol = re.compile(r"\w+")
single_quote_string = re.compile(r"'((?:[^'\\]|\\')*)'")


class Argument(object):
    grammar = attr('key', Symbol), optional('=', attr('value', [number, symbol, single_quote_string]))


class Arguments(List):
    grammar = optional(csl([Argument]))


class Parameter(object):
    grammar = attr('key', Symbol), optional('=', attr('value', [number, symbol, single_quote_string]))


class Parameters(List):
    grammar = optional(csl([Parameter]))


class Assignment(object):
    grammar = 'var', attr('left', Symbol), '=', attr('right', [Symbol, single_quote_string])

    def __repr__(self):
        return 'assign({}, {})'.format(self.left, self.right)


class ForLoop(object):
    grammar = 'for', attr('var', Symbol), 'in', attr('iterator', Symbol), ':'

    def __repr__(self):
        return 'forloop({}, {})'.format(self.var, self.iterator)


class Def(object):
    grammar = 'def', name(), '(', attr('_parameters', Parameters), ')'

    @property
    def parameters(self):
        return [(i.key.name, getattr(i, 'value', None)) for i in self._parameters]


class CallArgument(object):
    grammar = attr(
        'value',
        [
            single_quote_string,
            number,
            (
                symbol,
                optional('=', [number, symbol, single_quote_string])
            )
        ]
    )


class CallArguments(List):
    grammar = optional(csl([CallArgument]))


class Call(object):
    grammar = name(), '(', attr('_arguments', CallArguments), ')'

    @property
    def arguments(self):
        result = []
        for i in self._arguments:
            value = i.value
            if isinstance(value, list):
                if len(value) == 1:
                    value = value[0]
                else:
                    value = tuple(value)
            result.append(value)
        return result


class Tag(List):

    grammar = (
        name(),
        attr('classes', maybe_some(('.', word))),
        attr('id', optional(('#', word))),
        optional('(', attr('args', Arguments), ')'),
        attr('content', restline),
        endl
    )

    def __repr__(self):
        result = self.name
        if self.classes:
            result += ''.join(['.{}'.format(i) for i in sorted(self.classes)])
        if self.id:
            result += '#{}'.format(self.id)
        if self.content:
            result += ' ...'
        return result


class Django(str):
    grammar = name(), attr('restline', restline)


class Include(object):
    grammar = 'include', attr('filename', single_quote_string)


class PegParser(object):
    """
    This class implements the parser interface for PyPUGly.

    Once the returning values are standardized one can implement the parser using another technology such as PLY.
    """

    def parse_tag(self, text):
        result = parse(text, Tag)
        return result

    def parse_code(self, text):
        result = parse(text, [Def, ForLoop, Assignment, Include])
        return result

    def parse_call(self, text):
        result = parse(text, Call)
        return result

    def parse_django(self, text):
        result = parse(text, Django)
        return result



from __future__ import unicode_literals
'''
Module for string manipulation functions
'''
import six


def dedent(text, ignore_first_linebreak=True, ignore_last_linebreak=True):
    """
    Heavily inspired by textwrap.dedent, with a few changes (as of python 2.7)
        - No longer transforming all-whitespace lines into ''
        - Options to ignore first and last linebreaks of `text`.

    The last option is particularly useful because of ESSS coding standards.
    For example, using the default textwrap.dedent to create a 3-line string would look like this:
        textwrap.dedent('''    line1
            line2
            line3'''
        )

    With these options, you can create a better looking code with:
        Dedent(
            '''
            line1
            line2
            line3
            '''
        )

    :param unicode text:
        Text to be dedented (see examples above)

    :param bool ignore_first_linebreak:
        If True, blank characters (\r\n\t ) up to the first '\n' is ignored

    :param bool ignore_last_linebreak:
        If True, black characters (\r\n\t ) after the last '\n' is ignored


    Original docs:
        Remove any common leading whitespace from every line in `text`.

        This can be used to make triple-quoted strings line up with the left edge of the display,
        while still presenting them in the source code in indented form.

        Note that tabs and spaces are both treated as whitespace, but they are not equal: the lines
        "  hello" and "\thello" are  considered to have no common leading whitespace.  (This
        behaviour is new in Python 2.5; older versions of this module incorrectly expanded tabs
        before searching for common leading whitespace.)
    """
    if ignore_first_linebreak and '\n' in text:
        first, others = text.split('\n', 1)
        if first.strip('\n\r\t ') == '':
            text = others
    if ignore_last_linebreak and '\n' in text:
        others, last = text.rsplit('\n', 1)
        if last.strip('\n\r\t ') == '':
            text = others

    import re
    _leading_whitespace_re = re.compile('(^[ ]*)(?:[^ \n])', re.MULTILINE)

    # Look for the longest leading string of spaces and tabs common to
    # all non-empty lines.
    margin = None
    indents = _leading_whitespace_re.findall(text)

    for indent in indents:
        if margin is None:
            margin = indent

        # Current line more deeply indented than previous winner:
        # no change (previous winner is still on top).
        elif indent.startswith(margin):
            pass

        # Current line consistent with and no deeper than previous winner:
        # it's the new winner.
        elif margin.startswith(indent):
            margin = indent

    if margin:
        text = re.sub(r'(?m)^' + margin, '', text)
    return text


def indent(text, indent=1, indentation='    '):
    """
    Indents multiple lines of text.

    :param list(unicode)|unicode text:
        The text to apply the indentation.

    :param int indent:
        Number of indentations to add. Defaults to 1.

    :param unicode indentation:
        The text used as indentation. Defaults to 4 spaces.

    :return unicode:
        Returns the text with applied indentation.
    """
    indentation = indent * indentation

    lines = text
    if isinstance(lines, six.text_type):
        append_eol = lines.endswith('\n')
        lines = lines.splitlines()
    else:
        append_eol = True

    result = []
    for i in lines:
        if i.strip():
            result.append(indentation + i)
        else:
            result.append(i)
    if result:
        result = '\n'.join(result)
        if append_eol:
            result += '\n'
    else:
        result = ''
    return result


def safe_split(s, sep, maxsplit=None, default='', reversed=False):
    """
    Perform a string split granting the size of the resulting list.

    :param unicode s: The input string.
    :param unicode sep: The separator.
    :param int maxsplit: The max number of splits. The len of the resulting len is granted to be maxsplit + 1
    :param default: The default value for filled values in the result.

    :return list(unicode):
        Returns a list with fixed size of maxsplit + 1.
    """
    # NOTE: Can't import "string" module for string.split/string.rsplit because of module name
    # clashing with this module.
    if reversed:
        split = lambda s,*args: s.rsplit(*args)
    else:
        split = lambda s,*args: s.split(*args)

    if maxsplit is None:
        result = split(s, sep)
    else:
        result = split(s, sep, maxsplit)
        result_len = maxsplit + 1
        diff_len = result_len - len(result)
        if diff_len > 0:
            defaults = [default] * diff_len
            if reversed:
                result = defaults + result
            else:
                result = result + defaults
    return result


def format_it(iterable, format_expr="'%s'"):
    """
    Formats an iterable into a string by applying format_expr to each item.

    The resulting string is equivalent to stringifying a list, but unicode
    items won't have the prefix 'u'.

    Ex:
    a = u'My Item'
    b = [a]
    FormatIterable(b) # outputs "['a']", rather than "[u'a']"

    :param object iterable:
    Any iterable object.
    :param unicode format_expr:
    The format expression to use on each item. Defaults to "'%s'" so that the
    string representation of each item is encapsulated in single quotes.
    """
    items = ', '.join((format_expr % (item,) for item in iterable))
    return '[%s]' % (items,)


def match_any(text, regexes):
    """
    Returns whether the given text matches any of the given regular expressions.

    :param unicode text: The text to check for match.
    :param list(unicode) regexes: List of regular expressions.
    :return boolean:
        Return True if the given text matches any of the given regular expressions.
    """
    import re
    for i_regex in regexes:
        if re.match(i_regex, text) != None:
            return True
    return False



# coding: UTF-8
from __future__ import unicode_literals
from pypugly.tag import create_tag


def test_tag():
    assert create_tag('alpha') == 'alpha'
    assert create_tag('alpha', id_='id_alpha') == 'alpha id="id_alpha"'
    assert create_tag('alpha', klass=['zulu', 'zebra']) == 'alpha class="zulu zebra"'



from invoke import run, task


@task
def release():
    """
    Publishes the project on PyPI.

    We have automatic publishing enabled on TRAVIS build, so this is not
    necessary... but I'll keep here for reference.
    """
    run("python setup.py sdist upload")


@task
def test():
    """
    Executes all the tests.
    """
    run("python setup.py pytest")


@task
def travis_setpass():
    """
    Stores the PyPI password (encrypted) in the .travis.yml file.
    """
    print("travis encrypt --add deploy.password")





from __future__ import unicode_literals, print_function
import pytest
from zerotk.easyfs import CreateFile

from pypugly._pypugly import generate


@pytest.mark.parametrize(
    'basename',
    [
        'smoke_test',
        'var',
        'def',
        'django',
        'def-defaults',
        'include',
        'attrs',
        'unicode',
    ]
)
def test_parser(embed_data, basename):
    input_filename = embed_data[basename + '.lang']
    obtained_filename = embed_data[basename + '.obtained.html']
    expected_filename = embed_data[basename + '.html']
    CreateFile(obtained_filename, generate(input_filename))
    embed_data.assert_equal_files(obtained_filename, expected_filename)


def test_format_arguments():
    from pypugly._pypugly import Function
    f = Function(
        'alpha',
        [('first', None), ('second', "'two'"), ('third', "'three'")],
        None
    )
    assert (
        f.format_arguments(
            ["'alpha'", "'bravo'", "'charlie'"]
        ) == {
            'first': 'alpha',
            'second': 'bravo',
            'third': 'charlie',
        }
    )
    assert (
        f.format_arguments(
            ["'alpha'"]
        ) == {
            'first': 'alpha',
            'second': 'two',
            'third': 'three',
        }
    )
    assert (
        f.format_arguments(
            ["'FIRST'", ('third', "'333'")]
        ) == {
            'first': 'FIRST',
            'second': 'two',
            'third': '333',
        }
    )



from __future__ import unicode_literals, print_function

import os

import six
from zerotk.easyfs import GetFileContents, GetFileLines, IsFile
from zerotk.easyfs._exceptions import FileNotFoundError
from zerotk.reraiseit import reraise

from pypugly.tag import create_tag
from ast import literal_eval



class LineToken(object):

    class EmptyToken(RuntimeError):
        pass

    INDENTATION = 4

    LINE_TYPE_ROOT = 'ROOT'
    LINE_TYPE_TAG = 'TAG'
    LINE_TYPE_CODE = 'CODE'
    LINE_TYPE_DJANGO = 'DJANGO'
    LINE_TYPE_COMMENT = 'COMMENT'
    LINE_TYPE_CALL = 'CALL'
    _LINE_TYPE_VALIDS = {
        '-': LINE_TYPE_CODE,
        '%': LINE_TYPE_DJANGO,
        '#': LINE_TYPE_COMMENT,
        '+': LINE_TYPE_CALL,
    }

    def __init__(self, line):
        self.indent, self.line_type, self.line = self._parse_line(line)
        self.children = []
        self.line_no = 0

    def _parse_line(self, line):
        if line == self.LINE_TYPE_ROOT:
            return 0, self.LINE_TYPE_ROOT, ''

        indent = len(line)
        line = line.lstrip()
        indent = int((indent - len(line)) / self.INDENTATION)

        if not line:
            raise LineToken.EmptyToken

        line_type = self._LINE_TYPE_VALIDS.get(line[0])
        if line_type:
            line = line[1:]
        else:
            line_type = self.LINE_TYPE_TAG

        return indent, line_type, line

    def __repr__(self):
        return '<{}:{} {}>'.format(self.__class__.__name__, self.line_type, self.line)

    @property
    def indentation(self):
        return ' ' * self.INDENTATION * self.indent


class PugParser(object):

    INDENT = '>>>'
    DEDENT = '<<<'

    def tokenize(self, text):

        def preprocess_text(txt):
            return txt.replace('\t', '    ').replace('\r', '')

        context = self.ParseContext()
        text = preprocess_text(text)
        for i, i_line in enumerate(text.split('\n')):
            assert len(context.current_token) > 0, 'Sanity check!'

            try:
                line_token = LineToken(i_line)
                line_token.line_no = i + 1
                context.build_tree(line_token)
            except LineToken.EmptyToken:
                pass

        return context.current_token[0]

    class ParseContext(object):

        def __init__(self):
            self.current_token = [LineToken(LineToken.LINE_TYPE_ROOT)]
            self.current_indent = 0

        def build_tree(self, token):
            # Handle token tree building.
            delta = token.indent - self.current_indent
            if delta > 0:
                assert delta == 1, 'Single indents (%s chars) only.' % LineToken.INDENTATION
            elif delta < 0:
                self.current_token.pop()
                for i in range(-delta):
                    self.current_token.pop()
            else:
                if len(self.current_token) > 1:
                    self.current_token.pop()
            parent = self.current_token[-1]
            parent.children.append(token)
            self.current_token.append(token)
            self.current_indent = token.indent


class Function(object):

    def __init__(self, name, parameters, code):
        """

        :param str name:
            The function name.
        :param list(tuple(str,str) parameters:
            List of parameters names and default values.
        :param ??? code:
        :return:
        """
        self.name = name
        self.parameters = parameters
        self.code = code

    def format_arguments(self, arguments):
        """
        Returns a dictionary with the arguments. Obtain arguments names from
        the function parameter.

        It handles positional and keywords arguments. It also handles default
        parameter values.

        :param list result:
            ?
        :return dict(str, object):
        """
        args = []
        kwargs = {}
        for i_argument in arguments:
            if isinstance(i_argument, tuple) and len(i_argument) == 2:
                kwargs[i_argument[0]] = i_argument[1]
            else:
                args.append(i_argument)

        result = {}
        for i, (i_parameter_name, i_default_value) in enumerate(self.parameters):
            try:
                arg = kwargs[i_parameter_name]
            except KeyError:
                try:
                    arg = args.pop(0)
                except IndexError:
                    if i_default_value is not None:
                        arg = i_default_value
                    else:
                        arg = ''
            result[i_parameter_name] = literal_eval(arg)

        return result


class HtmlGenerator(object):

    def __init__(self, include_paths):
        self.HANDLERS = {
            LineToken.LINE_TYPE_ROOT: self.handle_root,
            LineToken.LINE_TYPE_COMMENT: self.handle_comment,
            LineToken.LINE_TYPE_CODE: self.handle_code,
            LineToken.LINE_TYPE_DJANGO: self.handle_django,
            LineToken.LINE_TYPE_TAG: self.handle_tag,
            LineToken.LINE_TYPE_CALL: self.handle_call,
        }
        # Code variables (for now, later it will be more complex... with IFs
        # and FORs and expand_vars...
        self.functions = {}
        self.variables = {}
        self.include_paths = include_paths

        from pypugly.peg_parser import PegParser
        self.__parser = PegParser()

    def generate(self, line_token):
        """
        Generates HTML from a line_token (tree).

        :param LineToken line_token:
        :return str:
        """
        lines = self._handle_line_token(line_token, self.variables)
        return '\n'.join(lines)

    def _handle_line_token(self, t, context):
        handler = self.HANDLERS.get(t.line_type)
        assert \
            handler is not None, \
            'No handler for token of type "{}"'.format(t.line_type)

        try:
            result = handler(t, after=False, context=context)
            result += self._handle_children(t.children, context=context)
            result += handler(t, after=True, context=context)
        except Exception as e:
            reraise(e, 'While handling line-token {}'.format(six.text_type(t)))
        return result

    def _handle_children(self, children, context):
        result = []
        for i_child in children:
            result += self._handle_line_token(i_child, context=context)
        return result

    def handle_root(self, token, after, context):
        return []

    def handle_code(self, token, after, context):
        if after:
            return []

        code = self.__parser.parse_code(token.line)
        code_class = code.__class__.__name__

        if code_class == 'Assignment':
            self.variables[six.text_type(code.left)] = literal_eval(code.right)
        elif code_class == 'Def':
            self.functions[six.text_type(code.name)] = \
                Function(code.name, code.parameters, token.children)
            token.children = []
            return []
        elif code_class == 'Include':
            filename = self._eval(code.filename)
            parser = PugParser()
            input_contents = GetFileContents(self._find_file(filename))
            token_tree = parser.tokenize(input_contents)
            return self._handle_line_token(token_tree, self.variables)

        elif code_class == 'ForLoop':
            return [token.indentation + repr(code)]

        return []

    def handle_call(self, token, after, context):
        if after:
            return []

        # Parse function call...
        code = self.__parser.parse_call(token.line)

        # Obtain the associated function
        function = self.functions.get(six.text_type(code.name))
        assert function is not None

        # Prepare arguments
        arguments = function.format_arguments(code.arguments)

        # Prepare context for the function call, adding the global variables
        # and argument values
        context = self.variables
        context.update(arguments)

        # Call the function 'code'
        tokens = function.code[:]

        # Replaces the code indent with the function call indent.
        for i in tokens:
            i.indent -= 1

        return self._handle_children(tokens, context=context)

    def handle_django(self, token, after, context):

        if after:
            if not token.children:
                return []
            code = self.__parser.parse_django(token.line)
            end_tag = code.name
            end_tag = '{% end' + end_tag.strip() + ' %}'
            return [token.indentation + end_tag]
        else:
            code = self.__parser.parse_django(token.line)
            start_tag = code.name + ' ' + code.restline
            start_tag = '{% ' + start_tag.strip() + ' %}'
            return [token.indentation + start_tag]

    def handle_comment(self, token, after, context):
        if after:
            return []
        return []

    def handle_tag(self, token, after, context):

        # Parses the TAG line, extracting the id, classes, arguments and the
        # contents.
        try:
            tag = self.__parser.parse_tag(token.line)
        except Exception as e:
            reraise(e, 'While parsing tag in line %d' % token.line_no)
            raise

        result = []

        have_content = hasattr(tag, 'content') and tag.content
        have_children = len(token.children) > 0

        if have_content and have_children:
            raise RuntimeError('A tag should have contents OR children nodes.')

        if after:
            if not have_content and have_children:
                result.append(
                    token.indentation + '</{name}>'.format(
                        name=tag.name
                    )
                )
        else:
            args = getattr(tag, 'args', [])
            args = {
                i.key: self._eval(getattr(i, 'value', 'True'))
                for i in args
            }
            tag_text = create_tag(
                tag.name,
                args,
                klass=tag.classes,
                id_=tag.id
            )
            if have_content or have_children:
                tag_format = '<{}>'
            else:
                tag_format = '<{} />'
            line = token.indentation + tag_format.format(tag_text)

            if have_content:
                # With content, close the tag in the same line.
                end_tag = '</{}>'.format(tag.name)
                content = self._eval(tag.content)
                line += content + end_tag

            result.append(line)

        return result

    def _eval(self, literal):
        """
        Evaluates a literal.

        For now, we evaluate only text (unicode) or boolean values
        (True/False).

        :param str literal:
        :return object:
        """
        try:
            result = literal_eval(literal)
            if isinstance(result, bytes):
                result = result.decode('UTF-8')
            if isinstance(result, six.text_type):
                result = result.format(**self.variables)
            return result
        except Exception as e:
            reraise(e, 'While evaluation literal: "{}"'.format(literal))

    def _find_file(self, filename):
        filenames = []
        for i_include_path in self.include_paths:
            filenames.append(i_include_path + '/' + filename)

        for i_filename in filenames:
            if IsFile(i_filename):
                return i_filename

        raise FileNotFoundError(filename)


def generate(filename, include_paths=()):
    """
    Creates and HTML from the given PyPUGly filename.

    :param str filename:
    :param list(str) include_paths:
    :return str:
    """
    from pypugly._pypugly import PugParser, HtmlGenerator

    parser = PugParser()
    input_contents = GetFileContents(filename)
    token_tree = parser.tokenize(input_contents)
    generator = HtmlGenerator([os.path.dirname(filename)] + list(include_paths))
    return generator.generate(token_tree)



from __future__ import unicode_literals
import six


def create_tag(name, args=None, klass=[], id_=''):
    """

    Shortcut to create the contents of an open XML tag.
    :param str name:
    :param dict(str:str) args:
    :param list(str) klass:
    :param str id_:
    :return str:
    """
    result = name

    if args is None:
        args = {}

    # Add classes defined using dot syntax.
    if klass:
        lst = args.setdefault('class', [])
        lst += klass

    # Override "id" by the one defined using hash syntax.
    if id_:
        args['id'] = id_

    # Format tag arguments in alphabetical order.
    for i_name, i_value in sorted(args.items()):
        # Name without value
        if i_value is True:
            result += ' {}'.format(i_name)
            continue

        # List values: join with spaces
        if isinstance(i_value, list):
            value = ' '.join(i_value)
        else:
            value = i_value

        # Quote value
        if isinstance(value, six.text_type) and not (value.startswith("'") and value.endswith("'")):
            value = '"' + value + '"'
        result += ' {}={}'.format(i_name, value)

    return result



#!/bin/env python
from setuptools import setup


setup(
    name='pypugly',
    use_scm_version=True,

    author='Alexandre Andrade',
    author_email='kaniabi@gmail.com',

    url='https://github.com/zerotk/pypugly',

    description='Another HTML generator based on JADE.',
    long_description='''Another HTML generator based on JADE.''',

    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],

    include_package_data=True,

    packages=['pypugly'],

    keywords=['generator', 'html', 'jade'],

    install_requires=['pypeg2', 'zerotk.easyfs'],
    setup_requires=['setuptools_scm', 'pytest-runner'],
    tests_require=['coverage', 'pytest'],
)



from __future__ import unicode_literals, print_function
from pypugly.peg_parser import PegParser


def test_parse_tag():
    parser = PegParser()

    f = parser.parse_tag("tag")
    assert str(f) == '''tag'''
    assert not hasattr(f, 'args')

    f = parser.parse_tag("tag()")
    assert str(f) == '''tag'''
    assert len(f.args) == 0

    f = parser.parse_tag("tag(a=alpha, b=2, c='charlie')")
    assert str(f) == '''tag'''
    assert len(f.args) == 3
    arg1 = f.args[0]
    assert arg1.key == 'a'
    assert arg1.value == 'alpha'
    arg1 = f.args[1]
    assert arg1.key == 'b'
    assert arg1.value == '2'
    arg1 = f.args[2]
    assert arg1.key == 'c'
    assert arg1.value == "'charlie'"

    f = parser.parse_tag("tag.cls1()")
    assert str(f) == '''tag.cls1'''
    assert f.classes == ['cls1']
    assert f.id is None
    assert len(f.args) == 0

    f = parser.parse_tag("tag.cls1.cls2()")
    assert str(f) == '''tag.cls1.cls2'''
    assert f.classes == ['cls1', 'cls2']
    assert f.id is None
    assert len(f.args) == 0

    f = parser.parse_tag("tag#id1()")
    assert str(f) == '''tag#id1'''
    assert f.classes == []
    assert f.id == 'id1'
    assert len(f.args) == 0

    f = parser.parse_tag("tag.cls1.cls2#id1()")
    assert str(f) == '''tag.cls1.cls2#id1'''
    assert f.classes == ['cls1', 'cls2']
    assert f.id == 'id1'
    assert len(f.args) == 0


def test_parse_code():
    parser = PegParser()

    f = parser.parse_code("var alpha = 'bravo'")
    assert f.__class__.__name__ == 'Assignment'
    assert f.left == 'alpha'
    assert f.right == '\'bravo\''
    assert str(f) == '''assign(alpha, 'bravo')'''

    f = parser.parse_code("for i in 10:")
    assert f.__class__.__name__ == 'ForLoop'
    assert f.var == 'i'
    assert f.iterator == '10'
    assert str(f) == '''forloop(i, 10)'''

    f = parser.parse_code("def alpha(bravo, charlie)")
    assert f.__class__.__name__ == 'Def'
    assert f.name == 'alpha'
    assert f.parameters == [('bravo', None), ('charlie', None)]


def test_parse_call():
    parser = PegParser()

    f = parser.parse_call("function(alpha, bravo, charlie)")
    assert f.__class__.__name__ == 'Call'
    assert f.name == 'function'
    assert f.arguments == ['alpha', 'bravo', 'charlie']

    f = parser.parse_call("function('Alpha', 1)")
    assert f.__class__.__name__ == 'Call'
    assert f.name == 'function'
    assert f.arguments == ["'Alpha'", '1']

    f = parser.parse_call("function(1, bravo=2, charlie=3)")
    assert f.__class__.__name__ == 'Call'
    assert f.name == 'function'
    assert f.arguments == ['1', ('bravo', '2'), ('charlie', '3')]


def test_parse_django():
    parser = PegParser()

    f = parser.parse_django("include alpha.html")
    assert f.__class__.__name__ == 'Django'
    assert f.name == 'include'
    assert f.restline == 'alpha.html'

    f = parser.parse_django("include alpha.html bravo charlie.html delta=object.pk")
    assert f.__class__.__name__ == 'Django'
    assert f.name == 'include'
    assert f.restline == 'alpha.html bravo charlie.html delta=object.pk'
